package RNA::Hatena;
#=============================================================================
# Copyright (C) 2003           Ikki Ohmukai,      All rights reserved.
#
#
# This file is part of RNA.
#
#    RNA is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    RNA is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RNA; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#=============================================================================
# はてなアンテナのデータを格納するモジュール
#==============================================================================


use strict;
use RNA::RSS;
use RNA::Utils qw( :all );
use vars qw($VERSION $flag);

$VERSION = '0.1';

$flag = '__hatena__';


#=============================================================================
# サブルーチン
#=============================================================================
sub new {
    my $class = shift;
	my $self = {};
    
    bless $self, $class;
    
    $self->init(@_);
    
    return $self;
}

sub init {
    my $self = shift;
    my %hash = @_;

	$self->{URI} = $hash{URI};
	$self->{RSSArray} = [];
	$self->{Subject} = 'hatena';
	$self->{Title} = '';
}


# $self->parse_html(\$html);
sub parse_html
{
	my $self = shift;
	my $str_ref = shift;
	my $str = $$str_ref;
#	print STDERR "Content-type: text/plain\n\n$str"; exit;

	my @rss_array = ();

	# はてなアンテナのタイトル
	if($str =~ /\<h1\>(.+?)\<\/h1\>/si) {
		$self->{Title} = $1;
	}
	
	while($str =~ s/\<li\>(.+?)\s*\<a\s+href\=\"(\S+?)\"\>(.+?)\<\/a\>.*?\<p\s+class\=\"diff\"\>(.*?)\<\/p\>.*?\<\/li\>//si)
	{
		my $raw_date = $1;
		my $item_link = $2;
		my $site_title = $3;
		my $item_description = $4;
		my $item_title = $site_title;
		my $site_description = 'hatena entry';

		my ($y, $M, $d, $h, $m, $s) = ();
		if($raw_date =~ /(\d{4})\/(\d{2})\/(\d{2})\s+(\d{2}):(\d{2}):(\d{2})/){
			($y, $M, $d, $h, $m, $s) = ($1, $2, $3, $4, $5, $6);
		}
		# タイムゾーンは日本
		my $item_date = &dc_date($y, $M, $d, $h, $m, $s, '+09:00');

		# link
		my $site_link = '';
		if($item_link =~ /http\:\/\/a.hatena.ne.jp\/go\?(.+?)\d{14}/is){
			$site_link = $1;
		}

		my $rss = $self->generate_rss(
			$site_title, $site_link, $site_description, 
			$item_title, $item_link, $item_description, $item_date);

		push(@rss_array, $rss);
	}

	 $self->{RSSArray} = \@rss_array;
}


# $rss = $self->generate_rss($site_title, $site_link, $site_description, 
#	$item_title, $item_link, $item_description, $item_date);
#
sub generate_rss
{
	my $self = shift;
	my ($stitle, $slink, $sdescription, $ititle, $ilink, $idescription, $idate)
		= @_;

	my $subject = $self->{Subject};

	my $rss = new RNA::RSS;
	$rss->channel(
		title => $stitle,
		link => $slink,
		description => $sdescription,
		dc => {subject=>$subject}
	);

	$rss->{items}->[0] = {};
	my $item = $rss->{items}->[0];
	$item->{description} = $idescription;
	$item->{title} = $ititle;
	$item->{link} = $ilink;

	$item->{dc} = {};
	$item->{dc} = {date=>$idate, subject=>$subject};

	$rss->revise;

	return $rss;
}

#=============================================================================
1;

