#=============================================================================
# Copyright (C) 2003           Ikki Ohmukai,      All rights reserved.
#
#
# This file is part of RNA.
#
#    RNA is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    RNA is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RNA; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#=============================================================================
# ハイパーリンク，Trackback 関連のモジュール
#
# $self->{
#	trackback => {
#		'http://****/***.html' => $rss,
#		'http://...' => $rss2,
#		'http://...' => $rss3,
#		...
#	},
#	link => {
#		'http://****/***.html' => $rss,
#		'http://...' => $rss2,
#		'http://...' => $rss3,
#		...
#	}
# }
# 
# '対象コンテンツのURI （たいていは HTML）' 
#	=> 'コンテンツのリンク情報を格納するRSS （RNA::RSS）'
#
#=============================================================================
package RNA::LinkInfo;

use strict;
use Jcode;
use RNA::RSS;
use RNA::Utils qw( :all );
use vars qw($VERSION);

$VERSION = '0.1';

sub new {
	my $class = shift;
	my $self = {};
	
	bless $self, $class;
	
	$self->init(@_);
	
	return $self;
}

sub init {
	my $self = shift;
	my %hash = @_;

	$self->{trackback} = {};
}


# $trackback_hash_ref = $self->trackback_info;
sub trackback_info
{
	my $self = shift;
	return $self->{trackback};
}


# $self->build($uri_list_ref);
#	$uri_list_ref : array reference of uri of contents
#
sub build
{
	my $self = shift;
	my $uri_list_ref = shift;
	
	my @uri_list = @{$uri_list_ref};
	foreach my $uri (@uri_list ) {
		$self->add($uri);
	}
}

# $self->add($uri);
sub add
{
	my $self = shift;
	my $uri = shift;
	
# print STDERR $uri . "\n";
	my $rss = $self->get_link_info_rss($uri);

	if ($rss->{items}->[0]->{link}) {
#print STDERR $rss->{items}->[0]->{title}.' : '.$rss->{items}->[0]->{link}."\n";
		$self->add_item(uri=>$uri, rdf=>$rss);
	}
}

# $self->add_item(uri=>'http://***/***.html', rdf=>$rss)
sub add_item
{
	my $self = shift;
	my %hash = @_;

	my $uri = $hash{uri};
	my $rss = $hash{rdf};

	$self->{trackback}->{$uri} = $rss;
}


# $rss = $self->get_link_info_rss($uri);
#
#	RNA::RSS オブジェクトを返す
sub get_link_info_rss
{
	my $self = shift;
	my $uri = shift;

	# html 文書を取得
	my ($str, $headers) = &get_html($uri);
	Jcode::convert(\$str, 'utf8');	# UTF-8 に変換
	my $rss = new RNA::RSS;

	# trackback 情報の取得
	if($str =~ /\<rdf:Description.*?trackback:ping=\"(.+?)\".*?\/?\s*\>/si){
		my $tb_ping_uri = $1;
		# trackback CGI から情報取得
		my ($str, $headers) = &get_html($tb_ping_uri . '?__mode=rss');
		Jcode::convert(\$str, 'utf8');	# UTF-8 に変換

		$rss->parse($str);
		$self->set_trackback_flag($rss);
		$self->revise_rss($rss);
	}

	return $rss;
}


# $self->export_data($dir);
#
sub export_data
{
	my $self = shift;
	my $dir = shift;
	my %trackback = %{$self->{trackback}};

	$dir =~ s/\/$//;

	$self->clear_directory($dir);

	my $i = 1;
	foreach my $uri (keys %trackback) {
		my $rss = $trackback{$uri};
		
		my $filename = $dir . '/links_' . sprintf("%010d", $i++) . '.rdf';
		my $str = $rss->as_string;

		&print_file($filename, \$str);
	}
}


# $self->clear_directory($dir);
sub clear_directory
{
	my $self = shift;
	my $dir = shift;

	unlink(glob($dir . '/*.rdf'));
}


# $self->import_data($dir);
#
sub import_data
{
	my $self = shift;
	my $dir = shift;
	my %trackback = %{$self->{trackback}};

	$dir =~ s/\/$//;

	foreach my $filename (glob($dir . '/*.rdf')) {
		my $rss = new RNA::RSS;
		$rss->parsefile($filename);
		$self->set_trackback_flag($rss);
		$self->revise_rss($rss);

		my $uri = $rss->{channel}->{link};
		if($uri) {
			$self->add_item(uri=>$uri, rdf=>$rss);
		}
	}
}

# $self->revise_rss($rss);
sub revise_rss
{
	my $self = shift;
	my $rss = shift;
	foreach my $item (@{$rss->{items}}) {
		undef $item->{dc}->{source};
		undef $item->{dcterms}->{isPartOf};
	}
}


# $self->set_trackback_flag($rss);
sub set_trackback_flag
{
	my $self = shift;
	my $rss = shift;

	foreach my $item (@{$rss->{items}}) {
		$rss->add_node($item, (semblog=>{trackback=>$item->{link}}));
	}
}

#==============================================================================
1;
__END__

