package RNA::OPML;
#=============================================================================
# Copyright (C) 2003           Ikki Ohmukai,      All rights reserved.
#
#
# This file is part of RNA.
#
#    RNA is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    RNA is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RNA; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#=============================================================================
#
# OPML ֘ÃW[
#	http://www.opml.org/spec
#=============================================================================

use strict;
use Carp;
use RNA::Utils qw( :all );
use vars qw($VERSION);

$VERSION = '0.1';

my $default_version = '1.0';
my $default_encoding = 'UTF-8';

my %opml_field = (
	head => {
		title => '',
		dateCreated => '',  # Date and Time Specification of RFC 822.
		dateModified => '', # Date and Time Specification of RFC 822.

		ownerName => '',
		ownerEmail => '',

		# comma-separated list of line numbers that are expanded. 
		# The line numbers in the list tell you which headlines to expand. 
		# The order is important. For each element in the list, X, 
		# starting at the first summit, navigate flatdown X times and expand. 
		# Repeat for each element in the list.
		expansionState => '', 

		# number, saying which line of the outline is displayed on the top line
		#  of the window. This number is calculated with the expansion state 
		# already applied
		vertScrollState => '', 

		windowTop => '', 
		windowLeft => '',
		windowBottom => '',
		windowRight => '',
	},
	body => {
		outlines => []
		# outlines => [
		#	{
		#		title => '',  # category
		#		isComment => 'true', # whether commented
		#		isBreakpoint => 'false',
		#		outlines => [
		#			{	# each site
		#				type        => 'rss',
		#				text        => '', 
		#				title       => '',
		#				description => '',
		#				xmlUrl      => '' },
		#			{ ... 
		#								  }
		#		]
		#	}
		# ]
	}
);

#my $modules = {
#	'http://purl.org/rss/1.0/' => 'rss',
#};


sub new {
    my $class = shift;
	my $self = {};
    
    bless $self, $class;
    
    $self->init(@_);
    
    return $self;
}


sub init {
    my $self = shift;
    my %hash = @_;

	%{$self} = %opml_field;
	
    # modules
    $self->{modules} = {};

    #get version info
    (exists($hash{version}))
	? ($self->{version} = $hash{version})
	    : ($self->{version} = $default_version);

    # encoding
    (exists($hash{encoding}))
	? ($self->{encoding} = $hash{encoding})
	    : ($self->{encoding} = $default_encoding);
}

#sub add_module {
#    my $self = shift;
#    my $hash = {@_};
#
#    $hash->{prefix} =~ /^[a-z_][a-z0-9.-_]*$/ or
#    croak "a namespace prefix should look like [a-z_][a-z0-9.-_]*";
#
#    $hash->{uri} or
#    croak "a URI must be provided in a namespace declaration";
#
#	  $self->{modules}->{$hash->{uri}} = $hash->{prefix};
#}



sub as_string {
    my $self = shift;
    my $version = $self->{version};
    my $encoding = $self->{encoding};
    my $output = '';

	$output .= qq(<?xml version="1.0" encoding="$encoding" ?>
<opml version="$version">
);

	#----------------------
	# head
	#----------------------
	$output .= "\t<head>\n";

	my $head = $self->{head};
	foreach my $el (keys %{$head}) {
		if($head->{$el}) {
			$output .= "\t\t<$el>" . $head->{$el} . "</$el>\n";
		}
	}
	$output .= "\t</head>\n";

	#----------------------
	# body
	#----------------------
	$output .= "\t<body>\n";

	my $outlines = $self->{body}->{outlines};
	foreach my $outline (@{$outlines}) {
		$output .= $self->as_string_outline($outline);
	}
	$output .= "\t</body>\n";

	$output .= "</opml>\n";

    return $output;
}

# $str = $self->as_string_outline($outline);
sub as_string_outline {
    my $self = shift;
    my $outline = shift;

	my $output = '';
	my $attributes = '';
	my $content = '';
	foreach my $el (keys %{$outline}) {
#print STDERR $el . ' : ' . $outline->{$el} . "\n";
		my $val = $outline->{$el};
		if($el eq 'outlines') {
			# recursive
			foreach my $o (@{$val}) {
				$content .= $self->as_string_outline($o);
			}
		}
		elsif($val) {
			$attributes .= " $el=\"$val\"";
		}
	}

	if($content) {
		$output .= "\t\t<outline" . $attributes . " >\n" 
			. "\t$content\n"
			. "\t\t</outline>\n";
	}
	else {
		$output .= "\t\t<outline" . $attributes . " />\n";
	}

	return $output;
}

# $self->parse_rss($rss)
#
# 	$rss : RNA::RSS module
sub parse_rss {
	my $self = shift;
	$self->init((%$self));
	my $rss = shift;

	$self->parse_head($rss);
	$self->parse_body($rss);
}

# $self->parse_head($rss);
sub parse_head
{
	my $self = shift;
	my $rss = shift;
	my $dc = $rss->{channel}->{dc};

	my $date = '';
	if($dc->{date}) {
		$date = &dcdate_to_rfc822($dc->{date});
	}
	
	my %hash = (
		title => $rss->{channel}->{title},
		dateModified => $date ,
		ownerName => $dc->{creator}
	);

	my $head = $self->{head};
	foreach my $el (keys %{$head}) {
		if($hash{$el}){
			$head->{$el} = $hash{$el};
		}
	}
}

# $self->parse_body($rss);
sub parse_body
{
	my $self = shift;
	my $rss = shift;

	my @items = @{$rss->{items}};
	my %subject_hash = ();
	foreach my $item (@items){
		$subject_hash{$item->{dc}->{subject}} = 1;
	}
	my @subjects = sort keys %subject_hash;

	foreach my $subject (@subjects) {
		my $outline = {};
		$outline->{title} = $subject;
		$outline->{outlines} = [];
		
		foreach my $item (@items) {
			my $dc = $item->{dc};

			if($dc->{subject} ne $subject){
				next;
			}

#print STDERR $subject . ' : ' . $item->{link} . "\n";

			my $sub_outline = {
				type     => 'rss',
				text     => $item->{title} ? $item->{title} : '',
				title    => $item->{title} ? $item->{title} : '',
				description => $item->{description} ? $item->{description} :'',
				xmlUrl      => $item->{link} ? $item->{link} : '',
				creator => $dc->{creator} ? $dc->{creator} : ''
			};
			push (@{$outline->{outlines}}, $sub_outline);
		}

		$self->add_outline(%{$outline});
#print STDERR values %{$self->{body}->{outlines}->[-1]->{outlines}->[-1]} ;
#print "\n";
#print STDERR $self->{body}->{outlines}->[-1]->{title} . "\n";
	}
}

sub add_outline {
    my $self = shift;
    my $hash_ref = {@_};

	push (@{$self->{body}->{outlines}}, $hash_ref);
}




# $self->parse($str)
#
sub parse {
	my $self = shift;
	$self->init((%$self));

	my $str = shift;
}

sub parsefile {
	my $self = shift;
	my $file = shift;
	open(IN, $file) || croak "Cannot open file $file : $!";
	my $str = "";
	while(my $s = <IN>){
		$str .= $s;
	}
	$self->parse($str);
}


sub save {
    my ($self,$file) = @_;
    open(OUT,">$file") || croak "Cannot open file $file for write: $!";
    print OUT $self->as_string;
    close OUT;
}

# $result = &encode ($text);
sub encode {
	my $text = shift;
	
    $text =~ s/</&lt;/g;
    $text =~ s/>/&gt;/g;
    $text =~ s/"/&quot;/g;

	return $text;
}

1;
__END__

