#!/usr/bin/perl
#=============================================================================
# RNA Configuration 
#=============================================================================
#
# 設定ファイルの編集など，各種設定を行うスクリプト
#
#=============================================================================
# Copyright (C) 2003           Ikki Ohmukai,      All rights reserved.
#
#
# This file is part of RNA.
#
#    RNA is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    RNA is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RNA; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#=============================================================================

# ライブラリ
use strict;
use lib qw(lib extlib);
use CGI;
# use XML::RSS;
use RNA::RSS;
use RNA;
use RNA::Utils qw( :all );

my $cgi_query = new CGI;

#----------------------------------
# 基本設定
#----------------------------------
my $self_uri = $cgi_query->url(-relative=>1);

#==============================================================================
# main
#==============================================================================

my $rna = RNA->new(Config => './config.pl');
my $config = $rna->{Config};
my $list_rss_file = $config->{SiteList}->{File};
my $hatena_flag = $config->{HatenaFlag};


if($cgi_query->param('target') eq 'update_site_list'){

	my %hash = (
		action => $cgi_query->param('action'),

		select => $cgi_query->param('select'),
		link    => $cgi_query->param('link'),
		title  => $cgi_query->param('title'),
		description => $cgi_query->param('description'),
		creator => $cgi_query->param('creator'),
		subject => $cgi_query->param('subject')
	);
	
	foreach my $key ('title', 'description', 'creator', 'subject'){
		unless($hash{$key}){
			$hash{$key} = $config->{SiteList}->{EmptyContent};
		}
	}

	&update_site_list(%hash);
}
elsif($cgi_query->param('target') eq 'update_hatena'){
# はてなアンテナ
	my %hash = (
		action => $cgi_query->param('action'),

		link    => $cgi_query->param('link'),
		subject => $cgi_query->param('subject')
	);
	&update_hatena(%hash);
}
else{
	&show_config_page;
}


#=============================================================================
# サブルーチン
#=============================================================================

# &update_site_list(%hash);
sub update_site_list
{
	my %arg = @_;

#print "Content-type: text/plain\n\n";
#print $arg{link} . "\n";
#exit(1);

	# サイトリストの取り出し
	my $list_rss = new RNA::RSS;
	$list_rss->parsefile($list_rss_file);

	# 対象の <item> があるかどうか
	my %exist_hash = ();
	foreach my $item (@{$list_rss->{items}}){
		$exist_hash{$item->{link}} = 1;
	}

	if ($arg{action} =~ /delete/i){
	# delete
		# リストに無い場合，削除できない
		unless($exist_hash{$arg{link}}){
			&die_handler($arg{link} . 
				' : リストにありません．削除できません．');
		}

		my @new_items = ();
		foreach my $item (@{$list_rss->{items}}){
			if($item->{link} ne $arg{link}){
				push(@new_items, $item);
			}
		}
		$list_rss->{items} = \@new_items;
	}
	elsif($arg{action} =~ /add/i){
	# add
		# すでにある場合は，追加しない
		if($exist_hash{$arg{link}}){
			&die_handler($arg{link} . ' : すでにリストにあります．追加できません．');
		}

		# 追加処理
		$list_rss->add_item(
			title       => $arg{title},
			link        => $arg{link},
			description => $arg{description},
			dc => {
				subject => $arg{subject},
				creator => $arg{creator}
			}
		);
	}
	elsif($arg{action} =~ /update/i){
	# update
		# リストに無い場合，更新できない
		unless($exist_hash{$arg{select}}){
			&die_handler($arg{select} . ' : リストにありません．更新できません．');
		}
		
		foreach my $item (@{$list_rss->{items}}){
			if($item->{link} eq $arg{select}){
			#更新
				%{$item} = (
					title       => $arg{title},
					link        => $arg{link},
					description => $arg{description},
					dc => {
						subject => $arg{subject},
						creator => $arg{creator}
					}
				);

				last;
			}
		}
	}
	else{
		&die_handler($arg{action} . ' : 不正な操作');
	}

	#-------------------------------------
	# 出力
	#-------------------------------------
	&set_site_list($list_rss_file, $list_rss);
	
	#-----------------------------------
	# self_url へのリダイレクション
	#-----------------------------------
	print $cgi_query->redirect($self_uri);
}

#--------------------------------
# はてなアンテナ
#--------------------------------
# &update_hatena(%hash);
sub update_hatena
{
	my %arg = @_;
	my $new_link = $arg{link};
	my $subject = $config->{HatenaFlag};

	# サイトリストの取り出し
	my $list_rss = new RNA::RSS;
	$list_rss->parsefile($list_rss_file);
	my @items = @{$list_rss->{items}};

	# hatena の <item> の取り出し
	my $hatena_item = {};
	foreach my $i (0 .. $#items) {
		if($items[$i]->{dc}->{subject} eq $hatena_flag) {
			$hatena_item = splice(@items, $i, 1);
			last;
		}
	}
	$list_rss->{items} = \@items;

	# 更新
	$hatena_item->{title} = $config->{SiteList}->{EmptyContent};
	$hatena_item->{link} = $new_link;
	$hatena_item->{dc}->{subject} = $subject;
	if($hatena_item->{link}) {
		$list_rss->add_item(%{$hatena_item});
	}

	#-------------------------------------
	# 出力
	#-------------------------------------
	&set_site_list($list_rss_file, $list_rss);
	
	#-----------------------------------
	# self_url へのリダイレクション
	#-----------------------------------
	print $cgi_query->redirect($self_uri);
}


# &set_site_list($file, $rss)
sub set_site_list
{
	my ($file, $rss) = @_;

	&set_rss($rss, 
		$config->{SiteList}->{Title}, $config->{SiteList}->{URI}, 
		$config->{SiteList}->{Description}, $rna->{Date}, $config->{Owner}, 
		$config->{Language});
	my $str = $rss->as_string;
	&print_file($file, \$str);
}


#-----------------------------------
# configure ページ
#-----------------------------------
#	&show_config_page;
sub show_config_page
{
	# サイトリストの取り出し
	my $list_rss = new RNA::RSS;
	$list_rss->parsefile($list_rss_file);
	my @items = @{$list_rss->{items}};

	#------------------------------------
	# はてなアンテナの取り出し
	#------------------------------------
	my $hatena_item = {};
	foreach my $i (0 .. $#items) {
		if($items[$i]->{dc}->{subject} eq $hatena_flag) {
			$hatena_item = splice(@items, $i, 1);
			last;
		}
	}

	my $http_header = 
		$cgi_query->header(-type=>'text/html', 
			-charset => $config->{Encoding});

	my $indent = "\t\t";
	my $update_site_list_switch = join("\n$indent", 
		map {
			"case \"" . $_->{link} . "\" : \n" 
				. "$indent\tform.title.value = \"". $_->{title} . "\"\;\n"
				. "$indent\tform.link.value = \"". $_->{link} . "\"\;\n"
				. "$indent\tform.description.value = \"" 
					. $_->{description} . "\"\;\n"
				. "$indent\tform.creator.value = \"". $_->{dc}->{creator} 
					. "\"\;\n"
				. "$indent\tform.subject.value = \"". $_->{dc}->{subject} 
					. "\"\;\n"
				. "$indent\tbreak\;"
		} @items
	);

	$indent = "\t\t";
	my $update_site_list_options = join("\n$indent", 
		map {
			sprintf("<option value=\"%s\">%s \&nbsp\;\&nbsp\; : %s</option>", 
				$_->{link}, $_->{link}, $_->{title});
		} @items
	);

	#-------------------------
	# テンプレート
	#-------------------------
	my $template = &read_file($config->{Template}->{Configure});
	my %text_hash = %{$config->{Template}->{Element}};
	$text_hash{__RNAUpdateSiteListSwitch__} = $update_site_list_switch;
	$text_hash{RNASiteList}  = $list_rss_file;
	$text_hash{__RNAUpdateSiteListOptions__}  = $update_site_list_options;

	# はてなアンテナ
	$text_hash{__RNAHatenaAntennaURI__} = $hatena_item->{link};
	$text_hash{__RNAHatenaAntennaSubject__} = $hatena_item->{dc}->{subject};	
	foreach my $t (keys %text_hash){
		$template =~ s/\<\$$t\$\>/$text_hash{$t}/ges;
	}

	# 出力
	print $http_header;
	print $template;
}





