#!/usr/bin/perl
#=============================================================================
# RNA - RSS Semblog Antenna
#=============================================================================
#
# semblogのアンテナ
# semblog のリストは，rss 形式
#
#=============================================================================
# Copyright (C) 2003           Ikki Ohmukai,      All rights reserved.
#
#
# This file is part of RNA.
#
#    RNA is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    RNA is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RNA; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#=============================================================================

# ライブラリ
use strict;
use lib qw(lib extlib);
use CGI;
use RNA;
use RNA::RSS;
use RNA::OPML;

my $cgi = new CGI;

#----------------------------------
# 基本設定
#----------------------------------
my $self_uri = $cgi->self_url();

#==============================================================================
# main
#==============================================================================

# RNA オブジェクトの生成
my $rna = RNA->new(Config => './config.pl');
my $config = $rna->{Config};

# バックグラウンド処理
my $background = $cgi->param('background');
if($background) {
	$config->{Update}->{Background} = ($background eq 'yes') ? 1 : 0
}

#-----------------------------------
# モードの設定
#-----------------------------------
my $mode = $cgi->param('mode') ? $cgi->param('mode') : 'update';

#-----------------------------------
# リダイレト先のURI
#-----------------------------------
my $transfer_uri = $cgi->param('transfer') 
	? $cgi->param('transfer') : $cgi->referer() ;
$transfer_uri = 
	$transfer_uri ? $transfer_uri 
		: $rna->{Config}->{MainIndex}->{URI};
my $redirect = $cgi->redirect($transfer_uri);


#-----------------------------------
# 実行
#-----------------------------------
if($mode eq 'opml') {
	&opml;
}
elsif($mode eq 'cache') {
	$rna->cache(TransferURI => $transfer_uri, Response => $redirect);
}
else { # $mode eq 'update'
	$rna->run(Type=>'Update', TransferURI => $transfer_uri, Response => $redirect);
}

#=============================================================================
# サブルーチン
#=============================================================================

sub opml
{
	my $rss = new RNA::RSS;
	$rss->parsefile($config->{SiteList}->{File});
	my $opml = new RNA::OPML;
	$opml->parse_rss($rss);
	print $cgi->header(-type=>'text/xml', -charset => $config->{Encoding});
	print $opml->as_string;
}

